/*
Check the sys.configurations system table daily to calculate a baseline number for 
overall instance-wide configuration settings like memory, MAXDOP, etc.

KKline, Email at KEKline@sqlsentry.com, Social media at @kekline
Blog at http://blogs.sqlsentry.com/KevinKline 
*/

-- Demo the basic idea.
SELECT SUM(CAST(value_in_use AS INT)) -- value_in_use
FROM sys.configurations;


-- Insert the daily value into the DDLEvents table
INSERT msdb.dbo.DDLEvents
    (
        EventType,
        EventDDL,
        ObjectName
     )
    SELECT
        CAST('SP_CONFIGURE' AS NVARCHAR(64)),  
        SUM(CAST(value_in_use AS INT)),
        CAST('SYS.CONFIGURATIONS' AS NVARCHAR(64))
	FROM sys.configurations;



-- Create a job to check sys.configurations daily.
/****** Object:  Job [Check Server Config for Changes]    Script Date: 12/8/2015 9:49:45 AM ******/
BEGIN TRANSACTION
DECLARE @ReturnCode INT
SELECT @ReturnCode = 0
/****** Object:  JobCategory [Database Maintenance]    Script Date: 12/8/2015 9:49:45 AM ******/
IF NOT EXISTS (SELECT name FROM msdb.dbo.syscategories WHERE name=N'Database Maintenance' AND category_class=1)
BEGIN
EXEC @ReturnCode = msdb.dbo.sp_add_category @class=N'JOB', @type=N'LOCAL', @name=N'Database Maintenance'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

END

DECLARE @jobId BINARY(16)
EXEC @ReturnCode =  msdb.dbo.sp_add_job @job_name=N'Check Server Config for Changes', 
		@enabled=1, 
		@notify_level_eventlog=0, 
		@notify_level_email=2, 
		@notify_level_netsend=0, 
		@notify_level_page=0, 
		@delete_level=0, 
		@description=N'A daily check to ensure that no unauthorized changes have been made to the overall server settings.', 
		@category_name=N'Database Maintenance', 
		@owner_login_name=N'INTERCERVE\kkline', 
		@notify_email_operator_name=N'SuperAweomeDBA', @job_id = @jobId OUTPUT
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
/****** Object:  Step [Write Config Values]    Script Date: 12/8/2015 9:49:46 AM ******/
EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N'Write Config Values', 
		@step_id=1, 
		@cmdexec_success_code=0, 
		@on_success_action=1, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N'TSQL', 
		@command=N'INSERT msdb.dbo.DDLEvents
    (
        EventType,
        EventDDL,
        ObjectName
     )
    SELECT
        CAST(''SP_CONFIGURE'' AS NVARCHAR(64)),  
        SUM(CAST(value_in_use AS INT)),
        CAST(''SYS.CONFIGURATIONS'' AS NVARCHAR(64))
	FROM sys.configurations;', 
		@database_name=N'master', 
		@flags=4
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_update_job @job_id = @jobId, @start_step_id = 1
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobschedule @job_id=@jobId, @name=N'Nightly Config Check', 
		@enabled=1, 
		@freq_type=4, 
		@freq_interval=1, 
		@freq_subday_type=1, 
		@freq_subday_interval=0, 
		@freq_relative_interval=0, 
		@freq_recurrence_factor=0, 
		@active_start_date=20151208, 
		@active_end_date=99991231, 
		@active_start_time=100, 
		@active_end_time=235959, 
		@schedule_uid=N'0dde6e78-323b-49b1-9126-04ddd1cb7ba5'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
EXEC @ReturnCode = msdb.dbo.sp_add_jobserver @job_id = @jobId, @server_name = N'(local)'
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
COMMIT TRANSACTION
GOTO EndSave
QuitWithRollback:
    IF (@@TRANCOUNT > 0) ROLLBACK TRANSACTION
EndSave:

GO

